package industries.doublehelix.pwste;

import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.TreeMap;
import java.util.stream.Stream;

import org.apache.commons.lang3.StringUtils;

import net.jpountz.lz4.LZ4Factory;
import net.jpountz.lz4.LZ4FastDecompressor;

/**
 * Class to hold one timelapse (all images)
 */
public class TimeLapse {
	private Path sourcePath;
	private String rawName;
	private String processedName;
	private List<byte[]> images = new ArrayList<>();
	
	public TimeLapse(Path sourceDir) {
		sourcePath = sourceDir;
		
		this.rawName = sourceDir.getFileName().toString();
		if (!this.rawName.toLowerCase().endsWith("timelapse")) {
			throw new IllegalArgumentException("Not a timelapse save folder");
		}
		
		String nameParts[] = this.rawName.split("_");
		if (nameParts.length < 3) throw new IllegalArgumentException("Not a timelapse save folder");
		StringBuilder processedNameBuilder = new StringBuilder(StringUtils.capitalize(nameParts[0].toLowerCase()) + ":");
		// last one is always TIMELAPSE
		for (int i = 1; i < nameParts.length - 1; ++i) {
			if (i == 2) processedNameBuilder.append(" -"); // let's see if this'll hold up
			processedNameBuilder.append(" ").append(StringUtils.capitalize(nameParts[i].toLowerCase()));
		}
		this.processedName = processedNameBuilder.toString();
		//System.out.println("procname: " + processedName);
	}
	
	public synchronized void load() throws IOException {
		if (!images.isEmpty()) return; // already loaded
		
		TreeMap<Integer, Path> sortedFiles = new TreeMap<>();
		try (Stream<Path> filePathStream = Files.list(this.sourcePath)) {
			// the files are already numbered.. sadly not in a sortable way without some extra work or we'd get 0, 1, 100, 11
			filePathStream.forEach(filePath -> {
				String fileName = filePath.getFileName().toString();
				if (Files.isRegularFile(filePath) && fileName.toLowerCase().endsWith(".sav")) {
					String[] filenameParts = fileName.split("_");
					String numberStr = filenameParts[filenameParts.length - 1];
					// this may cause IndexOutOfBounds or NumberFormatException, but that means there's a problem with this 'lapse anyway
					numberStr = numberStr.substring(0, numberStr.length() - 4);
					sortedFiles.put(Integer.parseInt(numberStr), filePath);
				}
			});
		}
		for (int key : sortedFiles.keySet()) {
			Path saveFile = sortedFiles.get(key);
			images.add(getImageFromSaveFile(saveFile));
		}
	}
	
	private byte[] getImageFromSaveFile(Path saveFile) throws IOException {
		byte[] file = Files.readAllBytes(saveFile);
		
		// pure guess, but this may be the uncompressed size and then some if the save is not finished.
		//ByteBuffer uncompressedSizeWrap = ByteBuffer.wrap(Arrays.copyOfRange(file, 8, 12));
		//uncompressedSizeWrap.order(ByteOrder.LITTLE_ENDIAN);
		//int uncompressedSize = uncompressedSizeWrap.getInt();
		//System.out.println("UnCompressed size 1: " + uncompressedSize);

		ByteBuffer uncompressedSize2Wrap = ByteBuffer.wrap(Arrays.copyOfRange(file, 16, 20));
		uncompressedSize2Wrap.order(ByteOrder.LITTLE_ENDIAN);
		int uncompressedSize2 = uncompressedSize2Wrap.getInt();
		//System.out.println("UnCompressed size 2: " + uncompressedSize2);
		
		//ByteBuffer compressedSizeWrap = ByteBuffer.wrap(Arrays.copyOfRange(file, 12, 16));
		//compressedSizeWrap.order(ByteOrder.LITTLE_ENDIAN);
		//int compressedSize = compressedSizeWrap.getInt();
		//System.out.println("Compressed size: " + uncompressedSize);
		
		
		// skip 20 bytes
		//byte[] realFile = Arrays.copyOfRange(file, 20, file.length);
		//System.out.println(new String(realFile, StandardCharsets.UTF_8).substring(0, 50));
		
		LZ4Factory factory = LZ4Factory.fastestInstance();
		LZ4FastDecompressor decompressor = factory.fastDecompressor();
		byte[] restored = new byte[uncompressedSize2];
		decompressor.decompress(file, 20, restored, 0, uncompressedSize2);
		
		return restored;
	}
	

	public Path getSourcePath() {
		return sourcePath;
	}

	public String getRawName() {
		return rawName;
	}

	public String getProcessedName() {
		return processedName;
	}
	
	@Override
	public String toString() {
		return getProcessedName();
	}

	public List<byte[]> getImages() {
		return Collections.unmodifiableList(this.images);
	}
}
